@@ -35,7 +35,7 @@ SYNC_AUDIO = 6
 FILE_DOES_NOT_EXIST = 7
 IMAGE_SEQUENCE = 8
 
-# Mediaview filtering options
+# Media view filtering options
 SHOW_ALL_FILES = 0
 SHOW_VIDEO_FILES = 1
 SHOW_AUDIO_FILES = 2
@@ -43,7 +43,7 @@ SHOW_GRAPHICS_FILES = 3
 SHOW_IMAGE_SEQUENCES = 4
 SHOW_PATTERN_PRODUCERS = 5
 
-# Used to draw indicators that tell if more frames are available while trimming
+# These are used to draw indicators that tell if more frames are available while trimming
 ON_FIRST_FRAME = 0
 ON_LAST_FRAME = 1
 ON_BETWEEN_FRAME = 2
@@ -65,13 +65,14 @@ PROP_INT = 0
 PROP_FLOAT = 1
 PROP_EXPRESSION = 2
 
-# Display heights for tracks.
+# Display heights for tracks and timeline.
 TRACK_HEIGHT_NORMAL = 50 # track height in canvas and column
 TRACK_HEIGHT_SMALL = 25 # track height in canvas and column
-TRACK_HEIGHT_SMALLEST = 25 # maybe remove as it is no longer meaningful
+TRACK_HEIGHT_SMALLEST = 25 # maybe remove this as it is no longer used
+TLINE_HEIGHT = 260
 
 # Notebook widths
-NOTEBOOK_WIDTH = 600 # defines app min width with MONITOR_AREA_WIDTH
+NOTEBOOK_WIDTH = 600 # defines app min width together with MONITOR_AREA_WIDTH
 NOTEBOOK_WIDTH_WIDESCREEN = 500
 TOP_ROW_HEIGHT = 500
 
@@ -99,7 +100,7 @@ NON_MLT_PROPERTY = "propertynonmlt"
 MLT_SERVICE = "mlt_service"
 EXTRA_EDITOR = "extraeditor"
 
-# Available tracks max for flowblade
+# Available tracks maximum for Flowblade
 MAX_TRACKS = 9
 
 # Thumbnail image dimensions
@@ -117,7 +118,7 @@ SAVEFILE_VERSION = -1
 MIDBAR_COLOR = "#bdbdbd"
 
 # Media log event types
-MEDIA_LOG_ALL = -1 # no MediaLogEvent has this type, this used when filtering events for display
+MEDIA_LOG_ALL = -1 # no MediaLogEvent has this type, this is used when filtering events for display
 MEDIA_LOG_INSERT = 0
 MEDIA_LOG_MARKS_SET = 1
 
@@ -147,8 +148,8 @@ AUDIO_LEVELS_DIR = "audiolevels/"
 THUMBNAILS_DIR = "thumbnails"
 RENDERED_CLIPS_DIR = "rendered_clips"
 GMIC_DIR = "gmic"
-NODE_COMPOSITORS_DIR = "node_compositors"
-PHANTOM_DISK_CACHE_DIR = "phantom_disk_cache"
+PHANTOM_DIR = "phantom2d"
+PHANTOM_DISK_CACHE_DIR = "disk_cache"
 MATCH_FRAME_DIR = "match_frame"
 MATCH_FRAME = MATCH_FRAME_DIR + "/match_frame.png"
 MATCH_FRAME_NEW = MATCH_FRAME_DIR + "/match_frame_new.png"
@@ -169,7 +170,6 @@ MULTI_TRIM = 3
 # Jack options (not used currently)
 JACK_ON_START_UP_NO = 0
 JACK_ON_START_UP_YES = 1
-
 JACK_OUT_AUDIO = 0
 JACK_OUT_SYNC = 0
 
@@ -183,7 +183,7 @@ TRIM_VIEW_ON = 0
 TRIM_VIEW_SINGLE = 1
 TRIM_VIEW_OFF = 2
 
-# MIdbar layout
+# Midbar layout
 MIDBAR_TC_LEFT = 0
 MIDBAR_TC_CENTER = 1
 MIDBAR_COMPONENTS_CENTERED = 2
@@ -192,3 +192,29 @@ MIDBAR_COMPONENTS_CENTERED = 2
 SINGLE_WINDOW = 1
 TWO_WINDOWS = 2
 
+# Apr-2017 - SvdB
+SHORTCUTS_DEFAULT = 'Flowblade Default'
+SHORTCUTS_DEFAULT_XML = 'flowblade'
+SHORTCUTS_ROOT_TAG = 'flowblade'
+SHORTCUTS_TAG = 'shortcuts'
+
+# Project properties keys
+P_PROP_DISSOLVE_GROUP_FADE_IN = "P_PROP_DISSOLVE_GROUP_FADE_IN"
+P_PROP_DISSOLVE_GROUP_FADE_OUT = "P_PROP_DISSOLVE_GROUP_FADE_OUT"
+P_PROP_ANIM_GROUP_FADE_IN = "P_PROP_ANIM_GROUP_FADE_IN"
+P_PROP_ANIM_GROUP_FADE_OUT = "P_PROP_ANIM_GROUP_FADE_OUT"
+P_PROP_TLINE_SHRINK_VERTICAL = "tline_shrink_vertical"
+P_PROP_LAST_RENDER_SELECTIONS = "P_PROP_LAST_RENDER_SELECTIONS"
+P_PROP_TRANSITION_ENCODING = "P_PROP_TRANSITION_ENCODING"
+P_PROP_AUTO_FOLLOW = "P_PROP_AUTO_FOLLOW"
+
+# A context defining action taken when mouse press happens based on edit mode ands mouse position
+POINTER_CONTEXT_NONE = 0
+POINTER_CONTEXT_END_DRAG_LEFT = 1
+POINTER_CONTEXT_END_DRAG_RIGHT = 2
+POINTER_CONTEXT_COMPOSITOR_MOVE = 3
+POINTER_CONTEXT_COMPOSITOR_END_DRAG_LEFT = 4
+POINTER_CONTEXT_COMPOSITOR_END_DRAG_RIGHT = 5
+POINTER_CONTEXT_TRIM_LEFT = 6
+POINTER_CONTEXT_TRIM_RIGHT = 7
+POINTER_CONTEXT_BOX_SIDEWAYS = 8
