@@ -22,8 +22,8 @@
 Module handles saving and loading data that is related to the editor and not any particular project.
 """
 
-
 from gi.repository import Gtk
+from gi.repository import Gdk
 
 import os
 import pickle
@@ -32,6 +32,9 @@ import appconsts
 import mltprofiles
 import utils
 
+# Apr-2017 - SvdB
+import shortcuts
+
 PREFS_DOC = "prefs"
 RECENT_DOC = "recent"
 
@@ -170,20 +173,26 @@ def get_recent_projects():
 
 def update_prefs_from_widgets(widgets_tuples_tuple):
     # Unpack widgets
-    gen_opts_widgets, edit_prefs_widgets, view_prefs_widgets, performance_widgets = widgets_tuples_tuple
+    gen_opts_widgets, edit_prefs_widgets, playback_prefs_widgets, view_prefs_widgets, performance_widgets = widgets_tuples_tuple
 
     default_profile_combo, open_in_last_opened_check, open_in_last_rendered_check, undo_max_spin, load_order_combo = gen_opts_widgets
     
     # Jul-2016 - SvdB - Added play_pause_button
-    auto_play_in_clip_monitor_check, auto_center_check, grfx_insert_length_spin, \
-        trim_exit_click, trim_quick_enter, remember_clip_frame, overwrite_clip_drop, cover_delete, \
-        play_pause_button, mouse_scroll_action, hide_file_ext_button, auto_center_on_updown = edit_prefs_widgets
+    # Apr-2017 - SvdB - Added ffwd / rev values
+    gfx_length_spin, trim_exit_on_empty, trim_quick_enter, remember_clip_frame, \
+    overwrite_clip_drop, cover_delete, mouse_scroll_action, hide_file_ext_button = edit_prefs_widgets
     
-    use_english, disp_splash, buttons_style, dark_theme, theme_combo, audio_levels_combo, window_mode_combo, full_names = view_prefs_widgets
+    auto_play_in_clip_monitor_check, auto_center_check, play_pause_button, auto_center_on_updown, \
+    ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin = playback_prefs_widgets
+    
+    use_english, disp_splash, buttons_style, dark_theme, theme_combo, audio_levels_combo, window_mode_combo, full_names, double_track_hights = view_prefs_widgets
 
     # Jan-2017 - SvdB
     perf_render_threads, perf_drop_frames = performance_widgets
 
+    # Apr-2017 - SvdB
+    #shortcuts_combo = shortcuts_widgets
+
     global prefs
     prefs.open_in_last_opended_media_dir = open_in_last_opened_check.get_active()
     prefs.remember_last_render_dir = open_in_last_rendered_check.get_active()
@@ -193,8 +202,8 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
 
     prefs.auto_play_in_clip_monitor = auto_play_in_clip_monitor_check.get_active()
     prefs.auto_center_on_play_stop = auto_center_check.get_active()
-    prefs.default_grfx_length = int(grfx_insert_length_spin.get_adjustment().get_value())
-    prefs.empty_click_exits_trims = trim_exit_click.get_active()
+    prefs.default_grfx_length = int(gfx_length_spin.get_adjustment().get_value())
+    prefs.empty_click_exits_trims = trim_exit_on_empty.get_active()
     prefs.quick_enter_trims = trim_quick_enter.get_active()
     prefs.remember_monitor_clip_frame = remember_clip_frame.get_active()
     prefs.overwrite_clip_drop = (overwrite_clip_drop.get_active() == 0)
@@ -203,6 +212,10 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     prefs.play_pause = play_pause_button.get_active()
     prefs.hide_file_ext = hide_file_ext_button.get_active()
     prefs.mouse_scroll_action_is_zoom = (mouse_scroll_action.get_active() == 0)
+    # Apr-2017 - SvdB - ffwd / rev values
+    prefs.ffwd_rev_shift = int(ffwd_rev_shift_spin.get_adjustment().get_value())
+    prefs.ffwd_rev_ctrl = int(ffwd_rev_ctrl_spin.get_adjustment().get_value())
+    prefs.ffwd_rev_caps = int(ffwd_rev_caps_spin.get_adjustment().get_value())
     
     prefs.use_english_always = use_english.get_active()
     prefs.display_splash_screen = disp_splash.get_active()
@@ -217,6 +230,12 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     # Feb-2017 - SvdB - for full file names
     prefs.show_full_file_names = full_names.get_active()
     prefs.center_on_arrow_move = auto_center_on_updown.get_active()
+    prefs.double_track_hights = (double_track_hights.get_active() == 1)
+
+    #if prefs.shortcuts != shortcuts.shortcut_files[shortcuts_combo.get_active()]:
+    #    prefs.shortcuts = shortcuts.shortcut_files[shortcuts_combo.get_active()]
+    #    shortcuts.load_shortcuts()
+
 
 def get_graphics_default_in_out_length():
     in_fr = int(15000/2) - int(prefs.default_grfx_length/2)
@@ -237,7 +256,6 @@ def create_rendered_clips_folder_if_needed(user_dir):
             os.mkdir(render_folder + "/")
         prefs.render_folder = render_folder
 
-
 class EditorPreferences:
     """
     Class holds data of persistant user preferences for editor.
@@ -303,3 +321,11 @@ class EditorPreferences:
         # Feb-2017 - SvdB - for full file names
         self.show_full_file_names = False
         self.center_on_arrow_move = False
+        # Apr-2017 - SvdB - Using these values we maintain the original hardcoded speed
+        self.ffwd_rev_shift = 1
+        self.ffwd_rev_ctrl = 10
+        self.ffwd_rev_caps = 1
+        self.shortcuts = "flowblade.xml"
+        self.double_track_hights = False
+        self.delta_overlay = True
+        self.show_alpha_info_message = True
