@@ -25,6 +25,7 @@ Module handles keyevents.
 from gi.repository import Gtk
 from gi.repository import Gdk
 
+import appconsts
 import audiowaveform
 import clipeffectseditor
 import compositeeditor
@@ -42,6 +43,9 @@ import medialog
 import menuactions
 import monitorevent
 import mltrefhold
+# Apr-2017 - SvdB
+import shortcuts
+import re
 import tlineaction
 import tlinewidgets
 import trimmodes
@@ -61,9 +65,11 @@ def key_down(widget, event):
         if audiowaveform.waveform_thread != None:
             audiowaveform.waveform_thread.abort_rendering()
             return True
-        else:
-            if editorstate.current_is_move_mode() == False:
-                editevent.set_default_edit_mode()
+        elif editorstate.current_is_move_mode() == False:
+            editevent.set_default_edit_mode()
+            return True
+        elif gui.big_tc.get_visible_child_name() == "BigTCEntry":
+            gui.big_tc.set_visible_child_name("BigTCDisplay")
             return True
 
     # Compositor editors keyevents
@@ -150,14 +156,14 @@ def key_down(widget, event):
     return False
     
 def _timeline_has_focus():
-    if(gui.tline_canvas.widget.is_focus()
-       or gui.tline_column.widget.is_focus()
-       or gui.editor_window.modes_selector.widget.is_focus()
-       or (gui.pos_bar.widget.is_focus() and timeline_visible())
-       or gui.tline_scale.widget.is_focus()
+    if(gui.tline_canvas.widget.has_focus()
+       or gui.tline_column.widget.has_focus()
+       or gui.editor_window.modes_selector.widget.has_focus()
+       or (gui.pos_bar.widget.has_focus() and timeline_visible())
+       or gui.tline_scale.widget.has_focus()
        or glassbuttons.focus_group_has_focus(glassbuttons.DEFAULT_FOCUS_GROUP)):
         return True
-    
+        
     return False
     
 def _handle_tline_key_event(event):
@@ -166,117 +172,94 @@ def _handle_tline_key_event(event):
     Returns True for handled key presses to stop those
     keyevents from going forward.
     """
-    # I
-    if event.keyval == Gdk.KEY_i:
-        if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-            monitorevent.to_mark_in_pressed()
-            return True
+    # Apr-2017 - SvdB - For keyboard shortcuts
+    action = _get_shortcut_action(event)
+    
+    # Apr-2017 - SvdB - For ffwd / rev speeds
+    prefs = editorpersistance.prefs
+
+    if action == 'mark_in':
         monitorevent.mark_in_pressed()
         return True
-    if event.keyval == Gdk.KEY_I:
-        if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-            monitorevent.to_mark_in_pressed()
-            return True
+    if action == 'to_mark_in':
         monitorevent.to_mark_in_pressed()
         return True
-
-    # O
-    if event.keyval == Gdk.KEY_o:
-        if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-            monitorevent.to_mark_out_pressed()
-            return True
+    if action == 'zoom_out':
+        updater.zoom_out()
+    if action == 'zoom_in':
+        updater.zoom_in()
+    if action == 'mark_out':
         monitorevent.mark_out_pressed()
         return True
-    if event.keyval == Gdk.KEY_O:
-        if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-            monitorevent.to_mark_out_pressed()
-            return True
+    if action == 'to_mark_out':
         monitorevent.to_mark_out_pressed()
         return True
-
-    # SPACE
-    if event.keyval == Gdk.KEY_space:
+    if action == 'play_pause':
         if PLAYER().is_playing():
             monitorevent.stop_pressed()
         else:
             monitorevent.play_pressed()
         return True
-    
-    # TAB
-    if event.keyval == Gdk.KEY_Tab:
+    if action == 'switch_monitor':
         updater.switch_monitor_display()
         return True
-
-    # M
-    if event.keyval == Gdk.KEY_m:
+    if action == 'add_marker':
         tlineaction.add_marker()
         return True
-
-    # Number edit mode changes
-    if event.keyval == Gdk.KEY_1:
+    if action == 'edit_mode_insert':
         gui.editor_window.handle_insert_move_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_2:
+    if action == 'edit_mode_overwrite':
         gui.editor_window.handle_over_move_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_3:
+    if action == 'edit_mode_trim':
         gui.editor_window.handle_one_roll_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_4:
+    if action == 'edit_mode_roll':
         gui.editor_window.handle_two_roll_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_5:
+    if action == 'edit_mode_slip':
         gui.editor_window.handle_slide_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_6:
+    if action == 'edit_mode_spacer':
         gui.editor_window.handle_multi_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_7:
+    if action == 'edit_mode_box':
         gui.editor_window.handle_box_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-        
-    # X
-    if event.keyval == Gdk.KEY_x:
+    if action == 'cut':
         tlineaction.cut_pressed()
         return True
-
-    # G
-    if event.keyval == Gdk.KEY_g:
+    if action == 'log_range':
         medialog.log_range_clicked()
         return True
-
-    # R
-    if event.keyval == Gdk.KEY_r:
+    if action == 'toggle_ripple':
         gui.editor_window.toggle_trim_ripple_mode()
         return True
-
+    
+    
     # Key bindings for keyboard trimming
     if editorstate.current_is_active_trim_mode() == True:
-        # LEFT ARROW, prev frame
-        if event.keyval == Gdk.KEY_Left:
+        if action == 'prev_frame':
             trimmodes.left_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
             return True
-
-        # RIGHT ARROW, next frame
-        if event.keyval == Gdk.KEY_Right:
+        elif action == 'next_frame':
             trimmodes.right_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
             return True
-
-        if event.keyval == Gdk.KEY_Return:
+        elif action == 'enter_edit':
             trimmodes.enter_pressed()
             return True
 
     # Key bindings for MOVE MODES and _NO_EDIT modes
     if editorstate.current_is_move_mode() or editorstate.current_is_active_trim_mode() == False:
-         # UP ARROW, next cut
-        if event.keyval == Gdk.KEY_Up:
+        if action == 'next_cut':
             if editorstate.timeline_visible():
                 tline_frame = PLAYER().tracktor_producer.frame()
                 frame = current_sequence().find_next_cut_frame(tline_frame)
@@ -287,9 +270,7 @@ def _handle_tline_key_event(event):
                     return True
             else:
                 monitorevent.up_arrow_seek_on_monitor_clip()
-        
-        # DOWN ARROW, prev cut
-        if event.keyval == Gdk.KEY_Down:
+        if action == 'prev_cut':
             if editorstate.timeline_visible():
                 tline_frame = PLAYER().tracktor_producer.frame()
                 frame = current_sequence().find_prev_cut_frame(tline_frame)
@@ -301,87 +282,68 @@ def _handle_tline_key_event(event):
             else:
                  monitorevent.down_arrow_seek_on_monitor_clip()
                  return True
-            
-        # LEFT ARROW, prev frame
-        if event.keyval == Gdk.KEY_Left:
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                PLAYER().seek_delta(-10)
+        # Apr-2017 - SvdB - Add different speeds for different modifiers
+        # Allow user to select what speed belongs to what modifier, knowing that a combo of mods
+        # will MULTIPLY all speeds
+        # Available: SHIFT_MASK LOCK_MASK CONTROL_MASK
+        if action == 'prev_frame' or action == 'next_frame':
+            if action == 'prev_frame':
+                seek_amount = -1
             else:
-                PLAYER().seek_delta(-1)
-            return True
-
-        # RIGHT ARROW, next frame
-        if event.keyval == Gdk.KEY_Right:
+                seek_amount = 1
+                
+            if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
+                seek_amount = seek_amount * prefs.ffwd_rev_shift
             if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                PLAYER().seek_delta(10)
-            else:
-                PLAYER().seek_delta(1)
+                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
+            if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
+                seek_amount = seek_amount * prefs.ffwd_rev_caps
+            PLAYER().seek_delta(seek_amount)
             return True
-            
-        # T
-        if event.keyval == Gdk.KEY_t:
+        if action == '3_point_overwrite':
             tlineaction.three_point_overwrite_pressed()
             return True
-
-        # Y
-        if event.keyval == Gdk.KEY_y:
+        if action == 'insert':
             if not (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
                 tlineaction.insert_button_pressed()
                 return True
-
-        # U
-        if event.keyval == Gdk.KEY_u:
+        if action == 'append':
             tlineaction.append_button_pressed()
             return True
-
-        # J
-        if event.keyval == Gdk.KEY_j:
+        if action == 'slower':
             monitorevent.j_pressed()
             return True
-
-        # K
-        if event.keyval == Gdk.KEY_k:
+        if action == 'stop':
             monitorevent.k_pressed()
             return True
-
-        # L
-        if event.keyval == Gdk.KEY_l:
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                medialog.log_range_clicked()
-            else:
-                monitorevent.l_pressed()
+        if action == 'faster':
+            monitorevent.l_pressed()
             return True
-
-        # S
-        if event.keyval == Gdk.KEY_s:
+        if action == 'log_range':
+            medialog.log_range_clicked()
+            return True
+        if action == 'resync':
             tlineaction.resync_button_pressed()
             return True
-            
-        # DELETE
-        if event.keyval == Gdk.KEY_Delete:
+        if action == 'delete':
             # Clip selection and compositor selection are mutually exclusive, 
             # so max one one these will actually delete something
             tlineaction.splice_out_button_pressed()
             compositormodes.delete_current_selection()
-        
-        # HOME
-        if event.keyval == Gdk.KEY_Home:
+        if action == 'to_start':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             PLAYER().seek_frame(0)
             _move_to_beginning()
             return True
-
-        # END
-        if event.keyval == Gdk.KEY_End:
+        if action == 'to_end':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             PLAYER().seek_end()
             _move_to_end()
             return True
     else:
-        # HOME
-        if event.keyval == Gdk.KEY_Home:
+        if action == 'to_start':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             gui.editor_window.handle_insert_move_mode_button_press()
@@ -389,9 +351,7 @@ def _handle_tline_key_event(event):
             PLAYER().seek_frame(0)
             _move_to_beginning()
             return True
-
-        # END
-        if event.keyval == Gdk.KEY_End:
+        if action == 'to_end':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             gui.editor_window.handle_insert_move_mode_button_press()
@@ -404,110 +364,140 @@ def _handle_tline_key_event(event):
 
 
 def _handle_extended_tline_focus_events(event):
-    # This was added to fix to a bug long time ago but the rationale for "extended_tline_focus_events" has been forgotten, but probably still exists
+    # This function was added to fix to a bug long time ago but the rationale for "extended_tline_focus_events" has been forgotten, but probably still exists
+    # Apr-2017 - SvdB - For keyboard shortcuts
+    action = _get_shortcut_action(event)
+
+    # We're dropping monitor window in 2 window mode as part of timeline focus
     if not(_timeline_has_focus() or
-            gui.pos_bar.widget.is_focus() or
+            gui.pos_bar.widget.has_focus() or
             gui.sequence_editor_b.has_focus() or
             gui.clip_editor_b.has_focus()):
         return False
 
-    # T
-    if event.keyval == Gdk.KEY_t:
+    if action == '3_point_overwrite':
         tlineaction.three_point_overwrite_pressed()
         return True
-
-    # Y
-    if event.keyval == Gdk.KEY_y:
-        if not (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-            tlineaction.insert_button_pressed()
-            return True
-
-    # U
-    if event.keyval == Gdk.KEY_u:
+    if action == 'insert':
+        tlineaction.insert_button_pressed()
+        return True
+    if action == 'append':
         tlineaction.append_button_pressed()
         return True
-
-    # J
-    if event.keyval == Gdk.KEY_j:
+    if action == 'slower':
         monitorevent.j_pressed()
         return True
-
-    # K
-    if event.keyval == Gdk.KEY_k:
+    if action == 'stop':
         monitorevent.k_pressed()
         return True
-
-    # L
-    if event.keyval == Gdk.KEY_l:
-        if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-            medialog.log_range_clicked()
-        else:
-            monitorevent.l_pressed()
+    if action == 'faster':
+        monitorevent.l_pressed()
         return True
-
-    # TAB
-    if event.keyval == Gdk.KEY_Tab:
-        updater.switch_monitor_display()
-        return True
-
-    # G
-    if event.keyval == Gdk.KEY_g:
+    if action == 'log_range':
         medialog.log_range_clicked()
         return True
-        
-    # Number edit mode changes
-    if event.keyval == Gdk.KEY_1:
+    if action == 'switch_monitor':
+        updater.switch_monitor_display()
+        return True
+    if action == 'edit_mode_insert':
         gui.editor_window.handle_insert_move_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_2:
+    if action == 'edit_mode_overwrite':
         gui.editor_window.handle_over_move_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_3:
+    if action == 'edit_mode_trim':
         gui.editor_window.handle_one_roll_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_4:
+    if action == 'edit_mode_roll':
         gui.editor_window.handle_two_roll_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_5:
+    if action == 'edit_mode_slip':
         gui.editor_window.handle_slide_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_6:
+    if action == 'edit_mode_spacer':
         gui.editor_window.handle_multi_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
-    if event.keyval == Gdk.KEY_7:
+    if action == 'edit_mode_box':
         gui.editor_window.handle_box_mode_button_press()
         gui.editor_window.set_mode_selector_to_mode()
         return True
 
     return False
         
+# Apr-2017 - SvdB
+def _get_shortcut_action(event):
+    # Get the name of the key pressed
+    key_name = Gdk.keyval_name(event.keyval).lower()
+    # Check if this key is in the dictionary.
+    state = event.get_state()
+    # Now we have a key and a key state we need to check if it is a shortcut.
+    # If it IS a shortcut we need to determine what action to take
+    if key_name in shortcuts._keyboard_actions:
+        # Now get the associated dictionary
+        _secondary_dict = shortcuts._keyboard_actions[key_name]
+        # In order to check for all available combinations of Ctrl+Alt etc (CTRL+ALT should be the same as ALT_CTRL)
+        # we do a SORT on the string. So both CTRL+ALT and ALT+CTRL will become +ACLLRTT and can be easily compared
+        modifier = ""
+        if state & Gdk.ModifierType.CONTROL_MASK:
+            modifier = "CTRL"
+        if state & Gdk.ModifierType.MOD1_MASK:
+            if modifier != "":
+                modifier = modifier + "+"
+            modifier = modifier + "ALT"
+        if state & Gdk.ModifierType.SHIFT_MASK:
+            if modifier != "":
+                modifier = modifier + "+"
+            modifier = modifier + "SHIFT"
+        # CapsLock is used as an equivalent to SHIFT, here
+        if state & Gdk.ModifierType.LOCK_MASK:
+            if modifier != "":
+                modifier = modifier + "+"
+            modifier = modifier + "SHIFT"
+        # Set to None if no modifier found
+        if modifier == "":
+            modifier = 'None'
+        try:
+            action = _secondary_dict[''.join(sorted(re.sub('[\s]','',modifier.lower())))]
+        except:
+            try:
+                action = _secondary_dict[''.join(sorted(re.sub('[\s]','','Any'.lower())))]
+            except:
+                action = 'None'
+        return action
+    # We didn't find an action, so return nothing 
+    return 'None'
+
 def _handle_clip_key_event(event):
     # Key bindings for MOVE MODES
-    if editorstate.current_is_move_mode():                  
-        # LEFT ARROW, prev frame
-        if event.keyval == Gdk.KEY_Left:
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                PLAYER().seek_delta(-10)
+    if editorstate.current_is_move_mode():
+        action = _get_shortcut_action(event)
+        # Apr-2017 - SvdB - Add different speeds for different modifiers
+        # Allow user to select what speed belongs to what modifier, knowing that a combo of mods
+        # will MULTIPLY all speeds
+        # Available: SHIFT_MASK LOCK_MASK CONTROL_MASK
+        
+        prefs = editorpersistance.prefs
+        if action == 'prev_frame' or action == 'next_frame':
+            if action == 'prev_frame':
+                seek_amount = -1
             else:
-                PLAYER().seek_delta(-1)
+                seek_amount = 1
+            if (event.get_state() & Gdk.ModifierType.SHIFT_MASK):
+                seek_amount = seek_amount * prefs.ffwd_rev_shift
+            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
+                seek_amount = seek_amount * prefs.ffwd_rev_ctrl
+            if (event.get_state() & Gdk.ModifierType.LOCK_MASK):
+                seek_amount = seek_amount * prefs.ffwd_rev_caps
+            PLAYER().seek_delta(seek_amount)
             return True
 
-        # RIGHT ARROW, next frame
-        if event.keyval == Gdk.KEY_Right:
-            if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
-                PLAYER().seek_delta(10)
-            else:
-                PLAYER().seek_delta(1)
-            return True
-
-         # UP ARROW
-        if event.keyval == Gdk.KEY_Up:
+        if action == 'next_cut':
             if editorstate.timeline_visible():
                 tline_frame = PLAYER().tracktor_producer.frame()
                 frame = current_sequence().find_next_cut_frame(tline_frame)
@@ -519,9 +509,7 @@ def _handle_clip_key_event(event):
             else:
                  monitorevent.up_arrow_seek_on_monitor_clip()
                  return True
-        
-         # DOWN ARROW, prev cut
-        if event.keyval == Gdk.KEY_Down:
+        if action == 'prev_cut':
             if editorstate.timeline_visible():
                 tline_frame = PLAYER().tracktor_producer.frame()
                 frame = current_sequence().find_prev_cut_frame(tline_frame)
@@ -533,39 +521,21 @@ def _handle_clip_key_event(event):
             else:
                  monitorevent.down_arrow_seek_on_monitor_clip()
                  return True
-
-        # SPACE
-        if event.keyval == Gdk.KEY_space:
+        if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             else:
                 monitorevent.play_pressed()
-
-        # I
-        if event.keyval == Gdk.KEY_i:
-            if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-                monitorevent.to_mark_in_pressed()
-                return True
+        if action == 'mark_in':
             monitorevent.mark_in_pressed()
             return True
-        if event.keyval == Gdk.KEY_I:
-            if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-                monitorevent.to_mark_in_pressed()
-                return True
+        if action == 'to_mark_in':
             monitorevent.to_mark_in_pressed()
             return True
-
-        # O
-        if event.keyval == Gdk.KEY_o:
-            if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-                monitorevent.to_mark_out_pressed()
-                return True
+        if action == 'mark_out':
             monitorevent.mark_out_pressed()
             return True
-        if event.keyval == Gdk.KEY_O:
-            if (event.get_state() & Gdk.ModifierType.MOD1_MASK):
-                monitorevent.to_mark_out_pressed()
-                return True
+        if action == 'to_mark_out':
             monitorevent.to_mark_out_pressed()
             return True
 
@@ -617,6 +587,9 @@ def _handle_geometry_editor_keys(event):
             if kfeditor.get_focus_child() != None:
                 if kfeditor.__class__ == keyframeeditor.GeometryEditor or \
                 kfeditor.__class__ == keyframeeditor.RotatingGeometryEditor:
+                    # Apr-2017 - SvdB - For keyboard shortcuts. I have NOT changed the arrow keys for
+                    # the kfeditor action. That didn't seem appropriate
+                    action = _get_shortcut_action(event)
                     if ((event.keyval == Gdk.KEY_Left) 
                         or (event.keyval == Gdk.KEY_Right)
                         or (event.keyval == Gdk.KEY_Up)
@@ -625,7 +598,7 @@ def _handle_geometry_editor_keys(event):
                         return True
                     if event.keyval == Gdk.KEY_plus:
                         pass # not impl
-                    if event.keyval == Gdk.KEY_space:
+                    if action == 'play_pause':
                         if PLAYER().is_playing():
                             monitorevent.stop_pressed()
                         else:
@@ -634,9 +607,10 @@ def _handle_geometry_editor_keys(event):
     return False
 
 def _handle_effects_editor_keys(event):
+    action = _get_shortcut_action(event)
     focus_editor = _get_focus_keyframe_editor(clipeffectseditor.keyframe_editor_widgets)
     if focus_editor != None:
-      if event.keyval == Gdk.KEY_space:
+      if action == 'play_pause':
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             else:
