@@ -113,7 +113,9 @@ NON_ACTIVE_KF_ICON = None
 DISCONNECTED_SIGNAL_HANDLER = -9999999
 
 actions_menu = Gtk.Menu()
-        
+
+_shift_down = None
+
 # ----------------------------------------------------- editor objects
 class ClipKeyFrameEditor:
     """
@@ -319,7 +321,7 @@ class ClipKeyFrameEditor:
                     next_frame, val = self.keyframes[hit_kf + 1]
                     self.drag_max = next_frame - 1
                 except:
-                    self.drag_max = self.clip_length - 1
+                    self.drag_max = self.clip_length
             self.widget.queue_draw()
 
     def _motion_notify_event(self, x, y, state):
@@ -332,9 +334,6 @@ class ClipKeyFrameEditor:
             self._set_clip_frame(lx)
             self.parent_editor.clip_editor_frame_changed(self.current_clip_frame)
         elif self.current_mouse_action == KF_DRAG:
-            if abs(lx - self.drag_start_x) < KF_DRAG_THRESHOLD:
-                return
-            
             frame = self._get_drag_frame(lx)
             self.set_active_kf_frame(frame)
             self.current_clip_frame = frame
@@ -354,8 +353,6 @@ class ClipKeyFrameEditor:
             self.parent_editor.clip_editor_frame_changed(self.current_clip_frame)
             self.parent_editor.update_slider_value_display(self.current_clip_frame)
         elif self.current_mouse_action == KF_DRAG:
-            if abs(lx - self.drag_start_x) < KF_DRAG_THRESHOLD:
-                return
             frame = self._get_drag_frame(lx)
             self.set_active_kf_frame(frame)
             self.current_clip_frame = frame
@@ -805,7 +802,6 @@ class AbstractScreenEditor:
         self.keyframes.sort(key=_geom_kf_sort)
         
     def delete_active_keyframe(self, keyframe_index):
-        #print keyframe_index
         if keyframe_index == 0:
             # keyframe frame 0 cannot be removed
             return
@@ -878,13 +874,15 @@ class AbstractScreenEditor:
         delta_x = x - self.mouse_start_x
         delta_y = y - self.mouse_start_y
         
+        global _shift_down 
         if state & Gdk.ModifierType.SHIFT_MASK:
             if abs(x - self.mouse_start_x) < abs(y - self.mouse_start_y):
                 delta_x = 0
             else:
                 delta_y = 0
-        #elif state & Gdk.ModifierType.CONTROL_MASK:
-        #    print "control"
+            _shift_down = (self.mouse_start_x, self.mouse_start_y)
+        else:
+            _shift_down = None
                 
         self._shape__motion_notify_event(delta_x, delta_y, (state & Gdk.ModifierType.CONTROL_MASK))
 
@@ -893,8 +891,10 @@ class AbstractScreenEditor:
     def _shape__motion_notify_event(self, delta_x, delta_y, CTRL_DOWN):
         print "_shape__motion_notify_event not impl"
 
-    
     def _release_event(self, event):
+        global _shift_down 
+        _shift_down = None
+        
         if self.current_mouse_hit == NO_HIT:
             return
             
@@ -944,6 +944,17 @@ class AbstractScreenEditor:
                        self.coords.screen_w, self.coords.screen_h)
         cr.fill()
 
+        if _shift_down != None:
+            cr.set_source_rgb(0.0, 0.0, 0.77)
+            cr.set_line_width(1.0)
+            mx, my = _shift_down
+            cr.move_to(mx, 0)
+            cr.line_to(mx, h)
+            cr.stroke()
+            cr.move_to(0, my)
+            cr.line_to(w, my)
+            cr.stroke()
+            
         screen_rect = [self.coords.orig_x, self.coords.orig_y, 
                        self.coords.screen_w, self.coords.screen_h]
         self._draw_edge(cr, screen_rect)
@@ -1075,7 +1086,7 @@ class BoxGeometryScreenEditor(AbstractScreenEditor):
     def _draw_edit_shape(self, cr, allocation):
         # Edit rect is created here only when we're sure to have allocation
         if self.source_edit_rect == None:
-            self.source_edit_rect = EditRect(10, 10, 10, 10) # values are immediatyly overwritten
+            self.source_edit_rect = EditRect(10, 10, 10, 10) # values are immediately overwritten
             self._update_source_rect()
 
         # Draw source
@@ -1618,10 +1629,11 @@ class AbstractKeyFrameEditor(Gtk.VBox):
             self.clip_in = 0
 
         # Value slider
-        row, slider = guiutils.get_slider_row(editable_property, self.slider_value_changed)
+        row, slider, spin = guiutils.get_slider_row_and_spin_widget(editable_property, self.slider_value_changed)
         self.value_slider_row = row
         self.slider = slider
-        
+        self.spin = spin
+
         self.initializing = False # Hack against too early for on slider listner
 
     def display_tline_frame(self, tline_frame):
@@ -1769,13 +1781,22 @@ class KeyFrameEditor(AbstractKeyFrameEditor):
         self.queue_draw()
 
     def connect_to_update_on_release(self):
+        print "jjjjjj"
         self.editable_property.adjustment.disconnect(self.editable_property.value_changed_ID)
         self.editable_property.value_changed_ID = DISCONNECTED_SIGNAL_HANDLER
+        self.spin.connect("activate", lambda w:self.spin_value_changed(w))
+        self.spin.connect("button-release-event", lambda w, e:self.spin_value_changed(w))
         self.slider.connect("button-release-event", lambda w, e:self.slider_value_changed(w.get_adjustment()))
         
     def update_property_value(self):
         self.editable_property.write_out_keyframes(self.clip_editor.keyframes)
 
+    def spin_value_changed(self, w):
+        adj = w.get_adjustment()
+        val = int(w.get_text())
+        adj.set_value(float(val))
+        self.slider_value_changed(adj)
+
 
 class GeometryEditor(AbstractKeyFrameEditor):
     """
@@ -1988,6 +2009,7 @@ class GeometryEditor(AbstractKeyFrameEditor):
             view_size_index = 2
         self.geom_buttons_row.size_select.set_active(view_size_index)
 
+
 class RotatingGeometryEditor(GeometryEditor):
     
     def init_geom_gui(self, editable_property):
@@ -2000,37 +2022,9 @@ class RotatingGeometryEditor(GeometryEditor):
         self.geom_kf_edit.keyframe_parser = propertyparse.rotating_geom_keyframes_value_string_to_geom_kf_array
         self.geom_kf_edit.set_keyframes(editable_property.value, editable_property.get_in_value)
 
-def rotating_ge_write_out_keyframes(ep, keyframes):
-    x_val = ""
-    y_val = ""
-    x_scale_val = ""
-    y_scale_val = ""
-    rotation_val = ""
-    opacity_val = ""
-    
-    for kf in keyframes:
-        frame, transf, opacity = kf
-        x, y, x_scale, y_scale, rotation = transf
-        x_val += str(frame) + "=" + str(propertyparse.get_frei0r_cairo_position(x, ep.profile_width)) + ";"
-        y_val += str(frame) + "=" + str(propertyparse.get_frei0r_cairo_position(y, ep.profile_height)) + ";"
-        x_scale_val += str(frame) + "=" + str(propertyparse.get_frei0r_cairo_scale(x_scale)) + ";"
-        y_scale_val += str(frame) + "=" + str(propertyparse.get_frei0r_cairo_scale(y_scale)) + ";"
-        rotation_val += str(frame) + "=" + str(rotation / 360.0) + ";"
-        opacity_val += str(frame) + "=" + str(opacity / 100.0) + ";"
 
-    x_val = x_val.strip(";")
-    y_val = y_val.strip(";")
-    x_scale_val = x_scale_val.strip(";")
-    y_scale_val = y_scale_val.strip(";")
-    rotation_val = rotation_val.strip(";")
-    opacity_val = opacity_val.strip(";")
-   
-    ep.x.write_value(x_val)
-    ep.y.write_value(y_val)
-    ep.x_scale.write_value(x_scale_val)
-    ep.y_scale.write_value(y_scale_val)
-    ep.rotation.write_value(rotation_val)
-    ep.opacity.write_value(opacity_val)
+
+
 
 # ----------------------------------------------------------------- linear interpolation
 def _get_frame_value(frame, keyframes):
