@@ -43,7 +43,8 @@ RENDER_TICKER_DELAY = 0.05
 class Player:
     
     def __init__(self, profile):
-    
+        #self.consumer = None
+
         self.init_for_profile(profile)
         
         self.ticker = utils.Ticker(self._ticker_event, TICKER_DELAY)
@@ -73,6 +74,12 @@ class Player:
         """
         Creates consumer with sdl output to a gtk+ widget.
         """
+        # SDL 2 consumer is created after
+        #if editorstate.get_sdl_version() == editorstate.SDL_2:
+        #    print "refuse SDL1 consumer"
+        #    return
+
+        print "Create SDL1 consumer..."
         # Create consumer and set params
         self.consumer = mlt.Consumer(self.profile, "sdl")
         self.consumer.set("real_time", 1)
@@ -82,14 +89,39 @@ class Player:
 
         # Hold ref to switch back from rendering
         self.sdl_consumer = self.consumer 
+
+    """
+    def create_sdl2_video_consumer(self):
+
+        widget = gui.editor_window.tline_display
+        self.set_sdl_xwindow(widget)
         
+        # Create consumer and set params
+        self.consumer = mlt.Consumer(self.profile, "sdl")
+        self.consumer.set("real_time", 1)
+        self.consumer.set("rescale", "bicubic") # MLT options "nearest", "bilinear", "bicubic", "hyper"
+        self.consumer.set("resize", 1)
+        self.consumer.set("progressive", 1)
+        self.consumer.set("window_id", str(self.xid))
+        alloc = gui.editor_window.tline_display.get_allocation()
+        self.consumer.set("window_width", str(alloc.width))
+        self.consumer.set("window_height", str(alloc.height))
+        self.consumer.set("window_type", "widget")
+        self.consumer.set("renderer_type", "software")
+        # Hold ref to switch back from rendering
+        self.sdl_consumer = self.consumer 
+        
+        self.connect_and_start()
+    """
+    
     def set_sdl_xwindow(self, widget):
         """
         Connects SDL output to display widget's xwindow
         """
         os.putenv('SDL_WINDOWID', str(widget.get_window().get_xid()))
+        #self.xid = widget.get_window().get_xid()
         Gdk.flush()
-    
+
     def set_tracktor_producer(self, tractor):
         """
         Sets a MLT producer from multitrack timeline to be displayed.
@@ -104,7 +136,22 @@ class Player:
     def refresh(self): # Window events need this to get picture back
         self.consumer.stop()
         self.consumer.start()
-
+   
+        """
+        if self.consumer == None:
+            return 
+        if editorstate.get_sdl_version() == editorstate.SDL_2:
+            alloc = gui.editor_window.tline_display.get_allocation()
+            self.consumer.set("window_width", str(alloc.width))
+            self.consumer.set("window_height", str(alloc.height))
+            
+            self.consumer.stop()
+            self.consumer.start()
+        else:
+            self.consumer.stop()
+            self.consumer.start()
+        """
+        
     def is_stopped(self):
         return (self.producer.get_speed() == 0)
         
@@ -116,6 +163,9 @@ class Player:
         """
         Connects current procer and consumer and
         """
+        #if self.consumer == None:
+        #    return 
+
         self.consumer.purge()
         self.producer.set_speed(0)
         self.consumer.connect(self.producer)
@@ -241,6 +291,9 @@ class Player:
         return (self.producer.get_speed() != 0)
 
     def _ticker_event(self):
+        #if self.consumer == None:
+        #    return 
+            
         # Stop ticker if playback has stopped.
         if (self.consumer.is_stopped() or self.producer.get_speed() == 0):
             self.ticker.stop_ticker()
