@@ -22,7 +22,7 @@
 Module creates GUI editors for editable mlt properties.
 """
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, GObject
 
 import appconsts
 from editorstate import PROJECT
@@ -33,7 +33,9 @@ import guiutils
 import keyframeeditor
 import mltfilters
 import mlttransitions
+import propertyparse
 import translations
+import updater
 import utils
 
 EDITOR = "editor"
@@ -59,6 +61,7 @@ COLOR_BOX = "colorbox"                                      # One band color edi
 COLOR_LGG = "colorlgg"                                      # Editor for ColorLGG filter
 FILE_SELECTOR = "file_select"                               # File selector button for selecting single files from
 FILE_TYPES = "file_types"                                   # list of files types with "." chracters, like ".png.tga.bmp"
+FADE_LENGTH = "fade_length"                                 # Autofade composiyots fade length
 NO_EDITOR = "no_editor"                                     # No editor displayed for property
 
 COMPOSITE_EDITOR_BUILDER = "composite_properties"           # Creates a single row editor for multiple properties of composite transition
@@ -106,7 +109,6 @@ def get_filter_extra_editor_rows(filt, editable_properties):
     """
     Returns list of extraeditors GUI components.
     """
-    #print editable_properties
     extra_editors = filt.info.extra_editors
     rows = []
     for editor_name in extra_editors:
@@ -443,6 +445,49 @@ def _get_wipe_selector(editable_property):
     
     return editor_pane
 
+class FadeLengthEditor(Gtk.HBox):
+    def __init__(self, editable_property):
+
+        GObject.GObject.__init__(self)
+        self.set_homogeneous(False)
+        self.set_spacing(2)
+        
+        self.editable_property = editable_property
+        length = self.editable_property.clip.clip_out - self.editable_property.clip.clip_in + 1
+        
+        name = editable_property.get_display_name()
+        name = _p(name)
+        name_label = Gtk.Label(label=name + ":")
+        
+        label_box = Gtk.HBox()
+        label_box.pack_start(name_label, False, False, 0)
+        label_box.pack_start(Gtk.Label(), True, True, 0)
+        label_box.set_size_request(appconsts.PROPERTY_NAME_WIDTH, appconsts.PROPERTY_ROW_HEIGHT)
+           
+        self.spin = Gtk.SpinButton.new_with_range (1, 1000, 1)
+        self.spin.set_numeric(True)
+        self.spin.set_value(length)
+        self.spin.connect("value-changed", self.spin_value_changed)
+
+        self.pack_start(guiutils.pad_label(4,4), False, False, 0)
+        self.pack_start(label_box, False, False, 0)
+        self.pack_start(self.spin, False, False, 0)
+        self.pack_start(Gtk.Label(), True, True, 0)
+        
+    def spin_value_changed(self, spin):
+        if self.editable_property.clip.transition.info.name == "##auto_fade_in":
+            self.editable_property.clip.set_length_from_in(int(spin.get_value()))
+        else:
+            self.editable_property.clip.set_length_from_out(int(spin.get_value()))
+
+        updater.repaint_tline()
+    
+    def display_tline_frame(self, frame):
+        pass # we don't seem to need this afte all, panel gets recreated after cpompositor length change
+        
+def _get_fade_length_editor(editable_property):
+    return FadeLengthEditor(editable_property)
+
 def _wipe_preset_combo_changed(widget, ep, widgets):
     combo_box, use_preset_luma_combo, user_luma_select, user_luma_label, keys = widgets
     if widget.get_active() == 1:
@@ -547,49 +592,9 @@ def _compositor_editor_force_combo_box_callback(combo_box, data):
         deinterlace.write_value("1")
         progressive.write_value("1")
 
-def _create_rotion_geometry_editor(clip, editable_properties):
-    # Build a custom object that duck types for TransitionEditableProperty to use in editor
-    ep = utils.EmptyClass()
-    # pack real properties to go
-    ep.x = filter(lambda ep: ep.name == "x", editable_properties)[0]
-    ep.y = filter(lambda ep: ep.name == "y", editable_properties)[0]
-    ep.x_scale = filter(lambda ep: ep.name == "x scale", editable_properties)[0]
-    ep.y_scale = filter(lambda ep: ep.name == "y scale", editable_properties)[0]
-    ep.rotation = filter(lambda ep: ep.name == "rotation", editable_properties)[0]
-    ep.opacity = filter(lambda ep: ep.name == "opacity", editable_properties)[0]
-    # Screen width and height are needeed for frei0r conversions
-    ep.profile_width = current_sequence().profile.width()
-    ep.profile_height = current_sequence().profile.height()
-    # duck type methods, using opacity is not meaningful, any property with clip member could do
-    ep.get_clip_tline_pos = lambda : ep.opacity.clip.clip_in # clip is compositor, compositor in and out points staright in timeline frames
-    ep.get_clip_length = lambda : ep.opacity.clip.clip_out - ep.opacity.clip.clip_in + 1
-    ep.get_input_range_adjustment = lambda : Gtk.Adjustment(float(100), float(0), float(100), float(1))
-    ep.get_display_name = lambda : "Opacity"
-    ep.get_pixel_aspect_ratio = lambda : (float(current_sequence().profile.sample_aspect_num()) / current_sequence().profile.sample_aspect_den())
-    ep.get_in_value = lambda out_value : out_value # hard coded for opacity 100 -> 100 range
-    ep.write_out_keyframes = lambda w_kf : keyframeeditor.rotating_ge_write_out_keyframes(ep, w_kf)
-    # duck type members
-    x_tokens = ep.x.value.split(";")
-    y_tokens = ep.y.value.split(";")
-    x_scale_tokens = ep.x_scale.value.split(";")
-    y_scale_tokens = ep.y_scale.value.split(";")
-    rotation_tokens = ep.rotation.value.split(";")
-    opacity_tokens = ep.opacity.value.split(";")
+def _create_rotion_geometry_editor(clip, editable_properties):   
+    ep = propertyparse.create_editable_property_for_affine_blend(clip, editable_properties)
     
-    value = ""
-    for i in range(0, len(x_tokens)): # these better match, same number of keyframes for all values, or this will not work
-        frame, x = x_tokens[i].split("=")
-        frame, y = y_tokens[i].split("=")
-        frame, x_scale = x_scale_tokens[i].split("=")
-        frame, y_scale = y_scale_tokens[i].split("=")
-        frame, rotation = rotation_tokens[i].split("=")
-        frame, opacity = opacity_tokens[i].split("=")
-        
-        frame_str = str(frame) + "=" + str(x) + ":" + str(y) + ":" + str(x_scale) + ":" + str(y_scale) + ":" + str(rotation) + ":" + str(opacity)
-        value += frame_str + ";"
-
-    ep.value = value.strip(";")
-
     kf_edit = keyframeeditor.RotatingGeometryEditor(ep, False)
     return kf_edit
 
@@ -767,6 +772,7 @@ EDITOR_ROW_CREATORS = { \
     LADSPA_SLIDER: lambda ep: _get_ladspa_slider_row(ep),
     CLIP_FRAME_SLIDER: lambda ep: _get_clip_frame_slider(ep),
     FILE_SELECTOR: lambda ep: _get_file_select_editor(ep),
+    FADE_LENGTH: lambda ep: _get_fade_length_editor(ep),
     NO_EDITOR: lambda ep: _get_no_editor(),
     COMPOSITE_EDITOR_BUILDER: lambda comp, editable_properties: _create_composite_editor(comp, editable_properties),
     REGION_EDITOR_BUILDER: lambda comp, editable_properties: _create_region_editor(comp, editable_properties),
