@@ -23,12 +23,15 @@ This module handles track actions; mute, change active state, size change.
 """
 
 import appconsts
+import audiomonitoring
 import dialogutils
 import gui
 import guicomponents
 import editorstate
+import edit
 from editorstate import get_track
 from editorstate import current_sequence
+from editorstate import PROJECT
 import snapping
 import tlinewidgets
 import updater
@@ -83,7 +86,7 @@ def set_track_small_height(track_index):
     updater.repaint_tline()
     
 def mute_track(track, new_mute_state):
-    # NOTE: THIS IS A SAVED EDIT OF SEQUENCE, BUT IS NOT AN UNDOABLE EDIT
+    # NOTE: THIS IS A SAVED EDIT OF SEQUENCE, BUT IT IS NOT AN UNDOABLE EDIT.
     current_sequence().set_track_mute_state(track.id, new_mute_state)
     gui.tline_column.widget.queue_draw()
     
@@ -113,12 +116,19 @@ def _all_tracks_item_activated(widget, msg):
     if msg == "topactiveonly":
         _activate_only_current_top_active()
     
+    if msg == "shrink":
+         _tline_vertical_shrink_changed(widget)
+        
 def _tracks_resize_update():
     tlinewidgets.set_ref_line_y(gui.tline_canvas.widget.get_allocation())
     gui.tline_column.init_listeners()
     updater.repaint_tline()
     gui.tline_column.widget.queue_draw()
 
+def _tline_vertical_shrink_changed(widget):
+    PROJECT().project_properties[appconsts.P_PROP_TLINE_SHRINK_VERTICAL] = widget.get_active()
+    updater.set_timeline_height()
+
 def _activate_all_tracks():
     for i in range(0, len(current_sequence().tracks) - 1):
         current_sequence().tracks[i].active = True
@@ -137,6 +147,7 @@ def _activate_only_current_top_active():
 def audio_levels_menu_launch_pressed(widget, event):
     guicomponents.get_audio_levels_popup_menu(event, _audio_levels_item_activated)
 
+# THIS HANDLES MUCH MORE NOW, NAME _audio_levels_item_activated needs changing
 def _audio_levels_item_activated(widget, msg):
     if msg == "all":
         editorstate.display_all_audio_levels = True
@@ -149,6 +160,17 @@ def _audio_levels_item_activated(widget, msg):
         snapping.snapping_on = widget.get_active()
     elif msg == "magnet":
         snapping.show_magnet_icon = widget.get_active()
+    elif msg == "autofollow":
+        active = widget.get_active()
+        editorstate.auto_follow = active
+        PROJECT().set_project_property(appconsts.P_PROP_AUTO_FOLLOW, active)
+        if active == True:
+            # Do autofollow update if auto follow activated
+            compositor_autofollow_data = edit.get_full_compositor_sync_data()
+            edit.do_autofollow_redo(compositor_autofollow_data)
+        updater.repaint_tline()
+    elif msg == "pointer_sensitive_item":
+        editorstate.cursor_is_tline_sensitive = widget.get_active()
     else: # media thumbnails
         editorstate.display_clip_media_thumbnails = widget.get_active()
         updater.repaint_tline()
@@ -221,6 +243,8 @@ def track_center_pressed(data):
                     return 
             # Update track mute state
             current_sequence().set_track_mute_state(track.id, new_mute_state)
+            
+            audiomonitoring.update_mute_states()
             gui.tline_column.widget.queue_draw()
     
     if data.event.button == 3:
