@@ -23,6 +23,8 @@ Helper functions and data
 """
 import time
 
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 
 import math
@@ -30,6 +32,7 @@ import md5
 import os
 import re
 import threading
+import xml.dom.minidom
 
 import appconsts
 import editorstate
@@ -46,8 +49,8 @@ class Ticker:
     Calls function repeatedly with given delay between calls.
     """
     def __init__(self, action, delay):
-        self.action = action
-        self.delay = delay
+        self.action = action # callback function
+        self.delay = delay # in seconds
         self.running = False
         self.exited = False
     
@@ -81,7 +84,15 @@ class Ticker:
             event.wait(delay)
         self.exited = True
 
+
+class LaunchThread(threading.Thread):
+    def __init__(self, data, callback):
+        threading.Thread.__init__(self)
+        self.data = data
+        self.callback = callback
         
+    def run(self):
+        self.callback(self.data)
         
 # -------------------------------- UTIL FUNCTIONS
 def fps():
@@ -116,6 +127,46 @@ def get_tc_string(frame):
     """
     return get_tc_string_with_fps(frame, fps())
 
+def get_tc_string_short(frame):
+    tc_str = get_tc_string(frame)
+    while len(tc_str) > 4:
+        if tc_str[0: 1] == "0" or tc_str[0: 1] == ":":
+            tc_str = tc_str[1: len(tc_str)]
+        else:
+            break
+    return tc_str
+            
+def get_tc_frame(frame_str):
+    """
+    Return timecode frame from string
+    """
+    return get_tc_frame_with_fps(frame_str, fps())
+
+def get_tc_frame_with_fps(frame_str, frames_per_sec):
+    # split time string hh:mm:ss:ff into integer and
+    # calculate corresponding frame
+    try:
+        times = frame_str.split(":", 4)
+    except expression as identifier:
+        return 0
+
+    # now we calculate the sum of frames that would sum up at corresponding
+    # time
+    sum = 0
+    for t in times:
+        num = int(t)
+        sum = sum * 60 + num
+
+    # but well, actually, calculated sum is wrong, because according
+    # to our calculation, that would give us 60 fps, we need to correct that
+    # last 'num' is frames already, no need to correct those
+    sum = sum - num
+    sum = int(sum / (60.0 / round(frames_per_sec)))
+    sum = sum + num
+
+    # and that is our frame, so we return sum
+    return sum
+
 def get_tc_string_with_fps(frame, frames_per_sec):
     # convert fractional frame rates (like 23.976) into integers,
     # otherwise the timeline will slowly drift over time
@@ -302,6 +353,15 @@ def get_file_type(file_path):
     
     return "unknown"
 
+def is_mlt_xml_file(file_path):
+    name, ext = os.path.splitext(file_path)
+    ext = ext.lstrip(".")
+    ext = ext.lower()
+    if ext == "xml" or ext == "mlt":
+        return True
+    
+    return False
+        
 def hex_to_rgb(value):
     value = value.lstrip('#')
     lv = len(value)
@@ -356,15 +416,20 @@ def get_unique_name_for_audio_levels_file(media_file_path, profile):
     return file_name
     
 def get_hidden_user_dir_path():
+    """ 
+    this is nit for now, reactiva if flatpack snadboxing is tried.
+    
     if editorstate.use_xdg:
         return os.path.join( 
                             os.getenv("XDG_CONFIG_HOME", os.path.join(os.getenv("HOME"),".config")),
                             "flowblade/")
     else:
-        return os.getenv("HOME") + "/.flowblade/"
+    """
+    
+    return os.getenv("HOME") + "/.flowblade/"
 
 def get_phantom_disk_cache_folder():
-    return get_hidden_user_dir_path() +  appconsts.NODE_COMPOSITORS_DIR + "/" + appconsts.PHANTOM_DISK_CACHE_DIR
+    return get_hidden_user_dir_path() +  appconsts.PHANTOM_DIR + "/" + appconsts.PHANTOM_DISK_CACHE_DIR
 
 def get_hidden_screenshot_dir_path():
     return get_hidden_user_dir_path() + "screenshot/"
@@ -427,8 +492,31 @@ def get_file_producer_info(file_producer):
     info["progressive"] = frame.get_int("meta.media.progressive") == 1
     info["top_field_first"] = frame.get_int("meta.media.top_field_first") == 1
     
+    resource = clip.get("resource")
+    name, ext = os.path.splitext(resource)
+    ext = ext.lstrip(".")
+    ext = ext.lower()
+    if ext == "xml" or ext =="mlt":
+        update_xml_file_producer_info(resource, info)
+        
     return info
 
+def update_xml_file_producer_info(resource, info):
+    # xml and mlt files require reading xml file to determine producer info
+    mlt_doc = xml.dom.minidom.parse(resource)
+
+    mlt_node = mlt_doc.getElementsByTagName("mlt").item(0)
+    profile_node = mlt_node.getElementsByTagName("profile").item(0)
+
+    info["width"] = int(profile_node.getAttribute("width"))
+    info["height"] = int(profile_node.getAttribute("height"))
+    info["fps_num"] = float(profile_node.getAttribute("frame_rate_num"))
+    info["fps_den"] = float(profile_node.getAttribute("frame_rate_den"))
+    info["progressive"] = int(profile_node.getAttribute("progressive"))
+    
+    print info
+    #  <profile description="HD 720p 29.97 fps" width="1280" height="720" progressive="1" sample_aspect_num="1" sample_aspect_den="1" display_aspect_num="16" display_aspect_den="9" frame_rate_num="30000" frame_rate_den="1001" colorspace="0"/>
+    
 def is_media_file(file_path):
     file_type = get_file_type(file_path)
     if file_type == "unknown":
@@ -532,6 +620,7 @@ _video_file_extensions = [  "avi",
                             "m2ts",
                             "m2v",
                             "m4e",
+                            "mlt",
                             "mjpg",
                             "mp4v",
                             "mts",
@@ -560,7 +649,8 @@ _video_file_extensions = [  "avi",
                             "wmv",
                             "xvid",
                             "y4m",
-                            "yuv"]
+                            "yuv",
+                            "xml"]
                             
                             
 def start_timing(msg="start timing"):
